import React from 'react';
import {observer} from "mobx-react-lite";
import {CloudDownload} from "@material-ui/icons";
import {makeStyles} from "@material-ui/core/styles";
import {green, red} from "@material-ui/core/colors";
import {ButtonField} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {RxModuleComponentModel, RxModuleComponentStore} from "../../stores/RxModuleComponentStore";
import {CurrentUserModel, CurrentUserStore} from "@rx-suite/stores";
import {FormHelperText} from "@material-ui/core";
import {useSnackbar} from "notistack";

const useStyles = makeStyles(theme => ({
    enabledIcon: {
        color: green[500]
    },
    warningIcon: {
        color: red[500]
    },
    button: {
        marginBottom: theme.spacing(2.5) //todo figure out alignment issue with create button
    },
    wrapper: {
        position: 'relative',
        display: 'inline-block'
    },
    helperText: {
        position: 'relative',
        display: 'inline-block'
    }
}));

function reflect(promise: Promise<any>) {
    return promise.then(function (v: any) {
            return {v: v, status: "fulfilled"}
        },
        function (err: any) {
            return {error: err, status: "rejected"}
        });
}

export const UpdateAllButton: React.FunctionComponent = observer(() => {
    const classes = useStyles();
    const [loading, setLoading] = React.useState(false);
    const {enqueueSnackbar} = useSnackbar();

    const user = CurrentUserStore.selectedModel as CurrentUserModel;

    const allowedUpdates = (RxModuleComponentStore.models as RxModuleComponentModel[]).filter((model: RxModuleComponentModel) => {
        return model.updateAvailable && !model.is_updating && user && user.canModule('update', 'apply', model.module_id);
    });

    const onlineUpdates = allowedUpdates.filter((model: RxModuleComponentModel) => {
        return model.reachable;
    });

    const offlineUpdates = allowedUpdates.filter((model: RxModuleComponentModel) => {
        return !model.reachable;
    });

    const onClick = React.useCallback(async () => {
        setLoading(true);
        try {
            if (window.confirm(`${t`Perform update now?  The modules will be unavailable until the update is complete.  This may take several minutes.`} 
            
Affected modules: ${onlineUpdates.map(m => m.module_name).join(', ')}`)) {
                if (onlineUpdates.some(m => m.module_id === 'rx_hub')) {
                    enqueueSnackbar(t`Downloading & transferring non-Hub updates`, {variant: "info"});
                    await Promise.all(onlineUpdates.filter(m => m.module_id !== 'rx_hub').map(model => {
                        return reflect(model.update().then(resp => {
                            if (resp.error) {
                                enqueueSnackbar(`${model.module_name}: ${t`Update failed`} (${resp.error_details || ''})`, {variant: "error"})
                            } else {
                                enqueueSnackbar(`${model.module_name}: ${t`Update applying - the module may take several minutes to restart`}`, {variant: "success"})
                            }
                        }));
                    }));
                    enqueueSnackbar(t`Downloading Hub update`, {variant: "info"});
                    const hub = onlineUpdates.find(m => m.module_id === 'rx_hub');
                    hub!.update().then(resp => {
                        if (resp.error) {
                            enqueueSnackbar(`${hub!.module_name}: ${t`Update failed`} (${resp.error_details || ''})`, {variant: "error"})
                        } else {
                            enqueueSnackbar(`${hub!.module_name}: ${t`Update applying - the module may take several minutes to restart`}`, {variant: "success"})
                        }
                    });
                } else {
                    enqueueSnackbar(`${t`Downloading & transferring`}: ${onlineUpdates.map(m => m.module_name).join(', ')}`, {variant: "info"});
                    onlineUpdates.forEach(model => {
                        model.update().then(resp => {
                            if (resp.error) {
                                enqueueSnackbar(`${model.module_name}: ${t`Update failed`} (${resp.error_details || ''})`, {variant: "error"})
                            } else {
                                enqueueSnackbar(`${model.module_name}: ${t`Update applying - the module may take several minutes to restart`}`, {variant: "success"})
                            }
                        });
                    })
                }
            }
        } finally {
            setLoading(false);
        }
    }, [enqueueSnackbar, onlineUpdates]);

    if (!user) {
        return null;
    }

    if (allowedUpdates.length === 0) {
        return <ButtonField
            className={classes.button}
            startIcon={<CloudDownload/>}
            text={t`All Up To Date`}
            variant={"outlined"}
            disabled
        />
    }
    let text, disabled;
    if (onlineUpdates.length > 0) {
        text = `${t`Update All`} (${onlineUpdates.map(m => m.module_name).join(', ')})`;
    } else {
        text = t`No applicable modules are online`;
        disabled = true;
    }
    return (
        <div className={classes.wrapper}>
            <ButtonField
                onClick={onClick}
                className={classes.button}
                startIcon={<CloudDownload className={disabled ? classes.warningIcon : classes.enabledIcon}/>}
                text={text}
                variant={"outlined"}
                loading={loading}
                disabled={disabled}
            />
            <div className={classes.helperText}>
                {(offlineUpdates.length > 0) && <FormHelperText error>
                    {t`The following modules are unreachable: `} {offlineUpdates.map(m => m.module_name).join(', ')}
                </FormHelperText>}
            </div>
        </div>
    )
});