import React from 'react';
import {observer} from "mobx-react-lite";
import {makeStyles} from "@material-ui/core/styles";
import {green, red} from "@material-ui/core/colors";
import {t} from "@lingui/macro";
import {RxModuleComponentModel} from "../../stores/RxModuleComponentStore";
import {useSnackbar} from 'notistack';
import {CurrentUserStore} from "@rx-suite/stores/build";
import {CurrentUserModel} from "@rx-suite/stores";
import {ButtonField} from "@rx-suite/ui/build";
import {CloudDownload} from "@material-ui/icons";

const useStyles = makeStyles(() => ({
    enabledIcon: {
        color: green[500]
    },
    warningIcon: {
        color: red[500]
    },
    errorLabel: {
        whiteSpace: 'pre-line',
        wordWrap: 'break-word'
    }
}))

type UpdateIconProps = {
    model: RxModuleComponentModel
}

export const UpdateCell: React.FunctionComponent<UpdateIconProps> = observer(({model}) => {
    const classes = useStyles();
    const {enqueueSnackbar} = useSnackbar();
    const onClick = React.useCallback(async e => {
        e.stopPropagation();
        if (window.confirm(`${t`Perform update now?  The module will be unavailable until the update is complete.  This may take several minutes.`} 
        
        ${t`Module:`} ${model.module_name} 
        ${t`Current Version:`} ${model.version}
        ${t`New Version:`} ${model.highestAvailableUpdate}`)) {
            enqueueSnackbar(`${model.module_name}: ${t`Update downloading & transferring`}`, {variant: "info"})
            model.update().then(resp => {
                if (resp.error) {
                    enqueueSnackbar(`${model.module_name}: ${t`Update failed`} (${resp.error_details || ''}) `, {variant: "error"})
                } else {
                    enqueueSnackbar(`${model.module_name}: ${t`Update applying - the module may take several minutes to restart`}    `, {variant: "success"})
                }
            });
        }
    }, [enqueueSnackbar, model]);

    const user = CurrentUserStore.selectedModel as unknown as CurrentUserModel;
    if (!user || !user.canModule('update', 'apply', model.module_id)) {
        return <div>{model.highestAvailableUpdate}</div>;
    }

    if (!model.updateAvailable) {
        return <div>{t`Up To Date`}</div>;
    }

    let text;
    if (model.reachable) {
        text = `${t`Update to`} ${model.highestAvailableUpdate}`;
    } else {
        text = `${t`[Offline]`} ${model.highestAvailableUpdate}`;
    }

    return (
        <ButtonField
            onClick={onClick}
            startIcon={<CloudDownload className={model.reachable ? classes.enabledIcon : classes.warningIcon}/>}
            text={text}
            variant={"outlined"}
            loading={model.is_updating}
            disabled={!model.reachable}
        />
    )
});