import * as React from "react";
import {observer} from "mobx-react-lite";
import Calendar from "@toast-ui/react-calendar";
import {ShiftStore} from "../../stores/ShiftStore";
import {asyncReactEffect} from "@rx-suite/ui";
import {ICalendarInfo, ISchedule} from "tui-calendar";
import {CalendarConfigStore} from "../../stores/CalendarConfigStore";
import {errToString, routerStore} from "@rx-suite/data";
import {CalendarCommandBar, ViewMode} from "./CalendarCommandBar";


export const ShiftsCalendar: React.FunctionComponent = observer(() => {
    const calendarRefContainer = React.useRef<Calendar>(null);
    const config = CalendarConfigStore.selectedModel;

    const [calendars, setCalendars] = React.useState<ICalendarInfo[]>([]);
    const [schedules, setSchedules] = React.useState<ISchedule[]>([]);
    const [view, setView] = React.useState<ViewMode>('week');
    const [cacheBuster, setCacheBuster] = React.useState(0);
    const [loading, setLoading] = React.useState(false);
    const [loaderr, setLoaderr] = React.useState('');

    const selectSchedule = React.useCallback(({schedule}) => {
        const link = ShiftStore.updateLink(schedule.calendarId);
        if (link) {
            routerStore.history.push(link);
        }
    }, []);
    const bustCache = React.useCallback(() => {
        setCacheBuster(i => i + 1);
    }, [setCacheBuster]);

    // eslint-disable-next-line react-hooks/exhaustive-deps
    React.useEffect(asyncReactEffect(
        () => {
            const calendar = calendarRefContainer.current as Calendar | undefined;
            if (calendar) {
                setLoading(true);
                setLoaderr('');
                return ShiftStore.getScheduleEvents(calendar.getInstance().getDateRangeStart().toDate(), calendar.getInstance().getDateRangeEnd().toDate());
            } else {
                return (async () => null)();
            }

        },
        (calendarResponse) => {
            if (calendarResponse) {
                setLoading(false);
                setLoaderr('');
                setCalendars(calendarResponse.map(scheduleData => ({
                    id: scheduleData.shift.getId().toString(),
                    name: scheduleData.shift.name as string,
                    bgColor: scheduleData.shift.color,
                    borderColor: scheduleData.shift.color
                })));
                setSchedules(calendarResponse.reduce((bucket, scheduleData) => {
                    scheduleData.dates.forEach((d, i) => {
                        bucket.push({
                            id: i.toString(),
                            calendarId: scheduleData.shift.getId().toString(),
                            title: scheduleData.shift.name,
                            category: 'time',
                            start: d.start.toISOString(),
                            end: d.end.toISOString()
                        })
                    });
                    return bucket;
                }, [] as ISchedule[]));
            }
        },
        err => {
            setLoading(false);
            setLoaderr(errToString(err));
        }
    ), [calendarRefContainer, setSchedules, setCalendars, setLoading, setLoaderr, cacheBuster, ShiftStore.lastWebsocketUpdate]);

    return (
        <>
            <CalendarCommandBar bustCache={bustCache} calendarRefContainer={calendarRefContainer} loaderr={loaderr}
                                loading={loading} setView={setView} view={view}
                                hasPendingChanges={ShiftStore.hasPendingChanges}/>
            <div style={{flexGrow: 1}}>
                <Calendar height={"100%"}
                          scheduleView={['time']}
                          taskView={false}
                          usageStatistics={false}
                          calendars={calendars}
                          schedules={schedules}
                          week={config?.weekConfig}
                          month={config?.monthConfig}
                          view={view}
                          ref={calendarRefContainer}
                          onClickSchedule={selectSchedule}
                          disableDblClick
                          disableClick
                          isReadOnly
                />
            </div>
        </>
    );
});