import * as React from "react";
import {observer} from "mobx-react-lite";
import Calendar from "@toast-ui/react-calendar";
import {ShiftModel, ShiftStore} from "../../../stores/ShiftStore";
import {asyncReactEffect} from "@rx-suite/ui";
import {ISchedule} from "tui-calendar";
import {CalendarConfigStore} from "../../../stores/CalendarConfigStore";
import {errToString} from "@rx-suite/data";
import moment from "moment";
import {CalendarCommandBar} from "../CalendarCommandBar";
import {t} from "@lingui/macro";

type viewModes = 'day' | 'week' | 'month';

export const PreviewCalendar: React.FunctionComponent<{ model: ShiftModel }> = observer(({model}) => {
        const calendarRefContainer = React.useRef<Calendar>(null);
        const config = CalendarConfigStore.selectedModel;

        const [schedules, setSchedules] = React.useState<ISchedule[]>([]);
        const [view, setView] = React.useState<viewModes>('week');
        const [cacheBuster, setCacheBuster] = React.useState(0);
        const [loading, setLoading] = React.useState(false);
        const [loaderr, setLoaderr] = React.useState('');

        const bustCache = React.useCallback(() => {
            setCacheBuster(i => i + 1);
        }, [setCacheBuster]);

        // eslint-disable-next-line react-hooks/exhaustive-deps
        React.useEffect(asyncReactEffect(
            () => {
                const calendar = calendarRefContainer.current as Calendar | undefined;
                if (calendar) {
                    setLoading(true);
                    setLoaderr('');
                    return ShiftStore.getPreviewEvents(calendar.getInstance().getDateRangeStart().toDate(), calendar.getInstance().getDateRangeEnd().toDate(), model.viewModel.shift_schedules);
                } else {
                    return (async () => null)();
                }

            },
            (calendarResponse: { start: moment.Moment, end: moment.Moment }[] | null) => {
                if (calendarResponse) {
                    setLoading(false);
                    setLoaderr('');
                    setSchedules(calendarResponse.map((d, i) => {
                        return {
                            id: i.toString(),
                            category: 'time',
                            start: d.start.toISOString(),
                            end: d.end.toISOString()
                        } as ISchedule
                    }));
                }
            },
            err => {
                setLoading(false);
                setLoaderr(errToString(err));
            }
        ), [calendarRefContainer, setSchedules, setLoading, setLoaderr, cacheBuster, model.viewModel.shift_schedules]);

        const schedulesWithModelData = schedules.map(s => ({
            ...s,
            calendarId: model.getId().toString(),
            title: model.viewModel.name || t`[Unnamed]`,
        }));
        return (
            <>
                <CalendarCommandBar bustCache={bustCache} calendarRefContainer={calendarRefContainer} loaderr={loaderr}
                                    loading={loading} setView={setView} view={view}/>
                <div style={{flexGrow: 1}}>
                    <Calendar height={`${(60 + (((CalendarConfigStore.selectedModel?.span as number) || 12) * 50))}px`}
                              scheduleView={['time']}
                              taskView={false}
                              usageStatistics={false}
                              calendars={[{
                                  id: model.getId().toString(),
                                  name: model.viewModel.name as string,
                                  bgColor: model.viewModel.color,
                                  borderColor: model.viewModel.color
                              }]}
                              schedules={schedulesWithModelData}
                              week={config?.weekConfig}
                              month={config?.monthConfig}
                              view={view}
                              ref={calendarRefContainer}
                              disableDblClick
                              disableClick
                              isReadOnly
                    />
                </div>
            </>
        );
    }
    )
;