import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {computed} from "mobx";
import {ComponentPermissionModel, ComponentPermissionStore} from "./ComponentPermissionStore";
import {RolePermissionModel, RolePermissionStore} from "./RolePermissionStore";

@model({
    transportLayer: "/api/roles",
    permissionTarget: "Role",
    eager: true,
    routingInfo: {
        create: '/roles/create',
        read: '/roles/:id/read',
        update: '/roles/:id/edit'
    }
})

export class RoleModel extends Model {
    @attribute({
        label: t`Name`,
        description: t`The name that will be used to identify this role.`,
        validation: yup.string().required(t`Name is a required field`)
    })
    name = '';

    get asJson(): Record<string, unknown> {
        return {
            ...this.toJSON(),
            role_permissions_attributes: this.permissions.map(rp => {
                rp.rolePermission.viewModel.submit();
                return rp.rolePermission.asJson
            })
        }
    }

    // Returns an array of all the component permissions and the associated role permissions
    @computed get permissions() {
        const roleRP = (RolePermissionStore.models || []).filter(rp => rp.role_id === this.id) as RolePermissionModel[];
        const componentPermissions = (ComponentPermissionStore.models || []) as ComponentPermissionModel[];
        //ensure every component permission has an matching role permission
        return componentPermissions.reduce((acc: Array<Permission>, cp: ComponentPermissionModel) => {
            let rp = roleRP.find((rp: RolePermissionModel) => cp.id === rp.component_permission_id);

            if (!rp) {
                rp = RolePermissionStore.createModel({
                    "component_permission_id": cp.id,
                    "permitted": false
                } as RolePermissionModel)
            }

            acc.push({componentPermission: cp, rolePermission: rp});

            return acc;
        }, []);
    }
}

export type Permission = {
    componentPermission: ComponentPermissionModel
    rolePermission: RolePermissionModel
}

export const RoleStore = RoleModel.store as unknown as DataStore<RoleModel>;
