class ComponentInstallJob < ApplicationJob
  queue_as :default

  def run_env
    # unsetenv_others: true breaks when calling bundler for unknown reasons ("Not a directory" error)
    %w[BUNDLER_ORIG_BUNDLER_VERSION BUNDLER_ORIG_BUNDLE_BIN_PATH BUNDLER_ORIG_BUNDLE_GEMFILE BUNDLER_ORIG_GEM_HOME BUNDLER_ORIG_GEM_PATH BUNDLER_ORIG_MANPATH BUNDLER_ORIG_PATH BUNDLER_ORIG_RB_USER_INSTALL BUNDLER_ORIG_RUBYLIB BUNDLER_ORIG_RUBYOPT BUNDLER_VERSION BUNDLE_BIN_PATH BUNDLE_GEMFILE GEM_HOME GEM_PATH RUBYLIB RUBYOPT PROD_DATABASE_URL DATABASE_URL].map do |k|
      [k, nil]
    end.to_h.merge({
      "RAILS_ENV" => "production",
      "PROD_DATABASE_URL" => db_url,
      "Path" => "#{RbConfig::CONFIG["bindir"]};#{ENV["Path"]}",
      "TZ" => ENV["TZ"]
    })
  end

  def db_url
    connection_info = ActiveRecord::Base.connection_config
    "#{connection_info[:adapter]}://#{connection_info[:username]}:#{connection_info[:password]}@#{connection_info[:host]}/#{@component.module_id}"
  end

  def run_command(cmd, working_dir)
    Rails.logger.info { "Running command: #{cmd} in #{working_dir}" }
    stdout, stderr, status = Open3.capture3(run_env,
      cmd,
      {chdir: working_dir})
    Rails.logger.info do
      {
        cmd: cmd,
        stdout: stdout,
        stderr: stderr,
        status: status
      }
    end
    raise "#{cmd} failed (#{stderr})" unless status.success?
  end

  def install_dir
    Rails.root.join("..", @component.module_id).to_s
  end

  def perform(component)
    ActiveRecord::Base.connection_pool.with_connection do
      @component = component
      Rails.logger.info { {action: "Installation starting", details: component.as_json} }
      component.mark_downloading_update

      url = "#{RxModuleComponent.update_server}/package/#{component.module_id}"
      url += "?prerelease=true" if ENV["ALLOW_PRERELEASE"]
      response = Faraday.get(url)
      if response.status == 200
        begin
          component.mark_update_step("Unpacking")
          Rails.logger.info { {action: "Extracting install package", details: component.as_json} }
          FileUtils.mkdir_p install_dir

          Tempfile.create(binmode: true) do |file|
            file.write(response.body)
            Zip.on_exists_proc = true
            Zip.validate_entry_sizes = false
            Zip::File.open(file) do |zip_file|
              zip_file.each do |entry|
                file_path = File.join(install_dir, entry.name)
                FileUtils.mkdir_p(File.dirname(file_path)) unless File.exist?(File.dirname(file_path))
                zip_file.extract(entry, file_path)
              end
            end
            file.close
          end
        rescue Errno::EACCES
          Rails.logger.warn { "Failed to delete tmpfile" }
        end

        Rails.logger.info { {action: "Generating .env", details: component.as_json} }
        component.mark_update_step("Configuring")
        File.open(File.join(install_dir, ".env"), "w") do |f|
          f.write "PROD_DATABASE_URL='#{db_url}'\n"
          f.write "RX_HUB_URL=http://#{Component.ensure_self&.ip_address || "127.0.0.1"}:#{Component.ensure_self&.port || ENV["MODULE_PORT"]}\n"
          f.write "MODULE_PORT=#{component.port}\n"
          f.write "TZ=#{ENV["TZ"]}\n" if ENV["TZ"]
        end

        bundle = "\"#{File.join(RbConfig::CONFIG["bindir"], "bundle").tr("/", "\\")}\""

        glob_path = File.join(install_dir, "vendor/cache/*.gem").tr("\\", "/")
        gem = "\"#{File.join(RbConfig::CONFIG["bindir"], "gem").tr("/", "\\")}\""
        # brute-force install gems regardless of dependency chain
        gems_to_check = Set.new(Dir.glob(glob_path))
        original_size = gems_to_check.length
        component.mark_update_step("Installing Dependencies")
        loop = true
        while loop
          loop = false
          size = gems_to_check.length
          installed = original_size - size
          component.mark_update_step("Installing Dependencies (#{installed}/#{original_size})")
          Rails.logger.debug { {action: "Iterating Gems", remaining: gems_to_check} }
          Dir.glob(glob_path).each do |gem_path|
            if (match = /(.+)-([0-9.]+).gem/.match(File.basename(gem_path))) && gems_to_check.include?(gem_path)
              Rails.logger.info { "Checking #{gem_path}" }
              if Gem::Specification.find_all_by_name(match[1], match[2]).length == 0
                begin
                  run_command("#{gem} install \"#{gem_path}\" --no-document", install_dir)
                  gems_to_check.delete(gem_path)
                  installed += 1
                  component.mark_update_step("Installing Dependencies (#{installed}/#{original_size})")
                  loop = true
                rescue => err
                  Rails.logger.debug { {action: "Installing Gem", gem: gem_path, err: err} }
                end
              else
                installed += 1
                component.mark_update_step("Installing Dependencies (#{installed}/#{original_size})")
                gems_to_check.delete(gem_path)
              end
            end
          end
          loop = false if size == gems_to_check.length
        end

        component.mark_update_step("Finalizing Dependencies")
        run_command("#{bundle} install --local  --gemfile \"#{File.join(install_dir, "Gemfile")}\"  --verbose", install_dir)
        run_command("#{bundle} pack", install_dir)
        component.mark_update_step("Setting up database & Windows service")
        run_command("#{bundle} exec rake rx_module:setup", install_dir)
        run_command("#{bundle} exec rake rx_module:start_service", install_dir)
        component.mark_update_step("Starting")
      else
        raise "Could not fetch install package from server"
      end
    rescue => err
      Rails.logger.warn { {action: "Failed to install", details: component.as_json} }
      component.update(last_update_error: err.message)
    ensure
      component.update(is_updating: false)
    end
  end
end
