# == Schema Information
#
# Table name: components
#
#  id                :bigint           not null, primary key
#  description       :string(4000)
#  has_token         :boolean
#  ip_address        :string(4000)
#  is_api            :boolean
#  is_updating       :boolean
#  last_update_error :string(4000)
#  module_name       :string(4000)
#  port              :integer
#  protocol          :string(4000)     default("http")
#  reachable         :boolean
#  token             :string(4000)
#  type              :string(4000)
#  update_status     :string(4000)
#  version           :string(4000)
#  created_at        :datetime         not null
#  updated_at        :datetime         not null
#  email_id          :bigint
#  module_id         :string(4000)
#
# Indexes
#
#  index_components_on_email_id             (email_id)
#  index_components_on_module_id            (module_id) UNIQUE
#  index_components_on_port_and_ip_address  (port,ip_address) UNIQUE
#
# Foreign Keys
#
#  fk_rails_...  (email_id => emails.id) ON DELETE => nullify
#
class Component < ApplicationRecord
  has_many :component_permissions
  belongs_to :email, optional: true

  validates_uniqueness_of :module_id, allow_nil: true
  validates_uniqueness_of :port, scope: :ip_address

  has_one_attached :icon_file
  has_secure_token

  class << self
    def hub_url
      @hub_url ||= find_by(module_id: "rx_hub")&.index
    end

    def display_name
      "Apps"
    end

    def ping
      ensure_self
      all.each do |component|
        component = component.becomes(component.type.constantize)
        component.ping
      end
    end

    def resolve(internal_id_or_module_id)
      if internal_id_or_module_id.is_a?(Integer) || /\b(?<!\.)\d+(?!\.)\b/ === internal_id_or_module_id
        begin
          find(internal_id_or_module_id)
        rescue ActiveRecord::RecordNotFound
          nil
        end
      else
        find_by(module_id: internal_id_or_module_id)
      end
    end

    def ensure_self
      RxModuleComponent.where(module_id: RxModule.module_id).first_or_create.tap do |mod|
        mod.module_name = RxModule.module_name
        if mod.ip_address.nil? || mod.ip_address.empty?
          mod.ip_address = ENV["RX_HUB_IP_ADDRESS"] || Socket.ip_address_list.detect(&:ipv4_private?).try(:ip_address) || "127.0.0.1"
        end
        mod.port = ENV["MODULE_PORT"] || (Rails.env.development? ? 3000 : 80)
        mod.save
      end
    end
  end

  def upgrade!
    raise "No generic upgrade implemented"
  end

  def to_json(*args)
    render_jbuilder_partial
  rescue
    super(*args)
  end

  def index
    "#{protocol || "http"}://#{ip_address}:#{port}"
  end

  def icon_url
    "#{Component.hub_url}/api/components/#{id}/icon"
  end

  def controller_name
    "ComponentsController"
  end

  def broadcast_class_name
    "RxModuleComponent"
  end

  def broadcast_local_name
    :component
  end

  def resolve_endpoint(endpoint)
    return index if endpoint.nil? || endpoint.empty?
    endpoint = endpoint.split("/").reject(&:empty?)
    "#{index}/#{endpoint.join("/")}"
  end

  def endpoint_authorization(endpoint)
    RxModule::Authorization.current_user_token
  end

  def mark_downloading_update
    update(is_updating: true, update_status: "Downloading package")
  end

  def mark_transferring_update
    update(is_updating: true, update_status: "Transferring package")
  end

  def mark_update_error(details)
    update(is_updating: false, update_status: nil, last_update_error: details)
  end

  def mark_update_applying
    update(reachable: false, version: "", last_update_error: nil, update_status: "Applying package")
  end

  def mark_update_step(msg)
    update(update_status: msg)
  end
end
