# == Schema Information
#
# Table name: shift_schedules
#
#  id              :bigint           not null, primary key
#  end_date        :datetime
#  recurrence_rule :string(4000)
#  shift_length    :integer
#  start_date      :datetime
#  created_at      :datetime         not null
#  updated_at      :datetime         not null
#  shift_id        :bigint           not null
#
# Indexes
#
#  index_shift_schedules_on_shift_id  (shift_id)
#
# Foreign Keys
#
#  fk_rails_...  (shift_id => shifts.id) ON DELETE => cascade
#
class ShiftSchedule < ApplicationRecord
  belongs_to :shift
  serialize :recurrence_rule, ::JSON

  def should_broadcast?
    false
  end

  class << self
    def rule_from_hash(opts)
      case opts["every"]
      when "day"
        IceCube::Rule.daily(opts["interval"] || 1)
      when "week"
        rule = IceCube::Rule.weekly(opts["interval"] || 1)
        if opts["on"]
          rule = rule.day(*opts["on"].map(&:to_sym))
        end
        rule
      when "month"
        rule = IceCube::Rule.monthly(opts["interval"] || 1)
        if opts["day"]
          rule = rule.day_of_week(opts["day"])
        elsif opts["mday"]
          rule = rule.day_of_month(*opts["mday"])
        end
        rule
      when "year"
        IceCube::Rule.yearly(opts["interval"] || 1)
      else
        raise "invalid recurrence #{opts["every"]}"
      end
    end

    def schedule_from_hash(start_date, duration, rule_hash)
      schedule = IceCube::Schedule.new(start_date, duration: duration)
      schedule.add_recurrence_rule rule_from_hash(rule_hash)
      schedule
    end

    def occurrences_between(start_date, end_date, duration, rule_hash)
      raise "Start and End Date are required to generate a schedule sequence" unless start_date && end_date # otherwise this method will run forever
      schedule_from_hash(start_date, duration, rule_hash).occurrences_between(start_date, end_date)
    end
  end

  def parsed_rule
    ShiftSchedule.rule_from_hash(recurrence_rule)
  end

  def parsed_schedule
    ShiftSchedule.schedule_from_hash(start_date, shift_length, recurrence_rule)
  end

  def occurrences_between(occurrence_start, occurrence_end)
    raise "Start and End Date are required to generate a schedule sequence" unless occurrence_start && occurrence_end # otherwise this method will run forever
    # we need to walk the start date back by the shift_length duration to capture events that begin before the start_date and run into the range
    # we need to cap the passed end date by the rule end date, if applicable
    parsed_schedule.occurrences_between((occurrence_start - shift_length.seconds), [occurrence_end, end_date].compact.min)
  end
end
